from tkinter import *
from VIStk.Objects._WindowGeometry import WindowGeometry
from VIStk.Objects._Window import Window

class Root(Tk, Window):
    """A wrapper for the Tk class with VIS attributes"""
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.WindowGeometry = WindowGeometry(self)
        self.Active = True
        self.protocol("WM_DELETE_WINDOW", self.unload)
        self.exitAction = None
        self.exitArgs = None
        self.exitKwargs = None
    
    def unload(self):
        """Closes the window neatly for VIStk"""
        for element in self.winfo_children():
            try:
                element.destroy()
            except: pass
        
        self.Active = False

    def exitQueue(self, action, *args, **kwargs):
        """Sets a function to call in the exit loop. Use for redirects."""
        self.exitAction = action
        self.exitArgs = tuple(*args)
        self.exitKwargs = {**kwargs}

    def exitAct(self):
        """Executes the exitAction"""
        if not self.exitAction is None:
            if not self.exitArgs is None:
                if not self.exitKwargs is None:
                    self.exitAction(tuple(self.exitArgs),self.exitKwargs)
                self.exitAction(tuple(self.exitArgs))
            else:
                if not self.exitKwargs is None:
                    self.exitAction(self.exitKwargs)
                else:
                    self.exitAction()


from VIStk.Structures.project import *
from VIStk.Structures.VINFO import *
from VIStk.Structures.screen import *
import subprocess
import shutil
from os.path import exists
import time
import datetime

info = {}

class Release():
    """A VIS Release object"""
    def __init__(self, project :Project, version:str="",type:str="",note:str=""):
        """Creates a Release object to release or examine a releaes of a project"""
        self.project = project
        self.version = version
        self.type = type
        self.note = note

    def build(self):
        """Build project spec file for release
        """
        
        print(f"Creating project.spec for {self.project.name}")

        with open(self.project.p_vinfo+"/Templates/spec.txt","r") as f:
            spec = f.read()
        with open(self.project.p_vinfo+"/Templates/collect.txt","r") as f:
            collect = f.read()
        
        spec_list = []
        name_list = []
        os.mkdir(self.project.p_vinfo+"/Build")
        for i in self.project.screenlist:
            if i.release:
                name_list.append(i.name)
                if not i.icon == None:
                    icon = i.icon
                else:
                    icon = self.project.d_icon
                spec_list.append(spec.replace("$name$",i.name))
                spec_list[len(spec_list)-1] = spec_list[len(spec_list)-1].replace("$icon$",icon)
                spec_list[len(spec_list)-1] = spec_list[len(spec_list)-1].replace("$file$",i.script)

                #build metadata
                with open(self.project.p_templates+"/version.txt","r") as f:
                    meta = f.read()

                #Update Overall Project Version
                vers = self.project.version.split(".")
                major = vers[0]
                minor = vers[1]
                patch = vers[2]
                meta = meta.replace("$M$",major)
                meta = meta.replace("$m$",minor)
                meta = meta.replace("$p$",patch)

                #Update Screen Version
                vers = i.s_version.split(".")
                major = vers[0]
                minor = vers[1]
                patch = vers[2]
                meta = meta.replace("$sM$",major)
                meta = meta.replace("$sm$",minor)
                meta = meta.replace("$sp$",patch)

                if self.project.company != None:
                    meta = meta.replace("$company$",self.project.company)
                    meta = meta.replace("$year$",str(datetime.datetime.now().year))
                else:
                    meta = meta.replace("            VALUE \"CompanyName\",      VER_COMPANYNAME_STR\n","")
                    meta = meta.replace("            VALUE \"LegalCopyright\",   VER_LEGALCOPYRIGHT_STR\n","")
                    meta = meta.replace("#define VER_LEGAL_COPYRIGHT_STR     \"Copyright © $year$ $company$\\0\"\n\n","")
                meta = meta.replace("$name$",i.name)
                meta = meta.replace("$desc$",i.desc)
                
                with open(self.project.p_vinfo+f"/Build/{i.name}.txt","w") as f:
                    f.write(meta)
                spec_list[len(spec_list)-1] = spec_list[len(spec_list)-1].replace("$meta$",self.project.p_vinfo+f"/Build/{i.name}.txt")
                spec_list.append("\n\n")

        insert = ""
        for i in name_list:
            insert=insert+"\n\t"+i+"_exe,\n\t"+i+"_a.binaries,\n\t"+i+"_a.zipfiles,\n\t"+i+"_a.datas,"
        collect = collect.replace("$insert$",insert)
        collect = collect.replace("$version$",self.project.name+"-"+self.version) if not self.version == "" else collect.replace("$version$",self.project.name)
        
        header = "# -*- mode: python ; coding: utf-8 -*-\n\n\n"

        with open(self.project.p_vinfo+"/project.spec","w") as f:
            f.write(header)
        with open(self.project.p_vinfo+"/project.spec","a") as f:
            f.writelines(spec_list)
            f.write(collect)

        print(f"Finished creating project.spec for {self.project.title} {self.version if not self.version =="" else "current"}")#advanced version will improve this

    def clean(self):
        """Cleans up build environment to save space
        """
        print("Cleaning up build environment")
        shutil.rmtree(self.project.p_vinfo+"/Build")
        print("Appending Screen Data To Environment")
        if self.version == " ":
            if exists(f"{self.project.p_project}/dist/{self.project.title}/Icons/"): shutil.rmtree(f"{self.project.p_project}/dist/{self.project.title}/Icons/")
            if exists(f"{self.project.p_project}/dist/{self.project.title}/Images/"): shutil.rmtree(f"{self.project.p_project}/dist/{self.project.title}/Images/")
            shutil.copytree(self.project.p_project+"/Icons/",f"{self.project.p_project}/dist/{self.project.title}/Icons/",dirs_exist_ok=True)
            shutil.copytree(self.project.p_project+"/Images/",f"{self.project.p_project}/dist/{self.project.title}/Images/",dirs_exist_ok=True)
        else:
            if exists(f"{self.project.p_project}/dist/{self.project.title}/Icons/"): shutil.rmtree(f"{self.project.p_project}/dist/{self.project.name}/Icons/")
            if exists(f"{self.project.p_project}/dist/{self.project.title}/Images/"): shutil.rmtree(f"{self.project.p_project}/dist/{self.project.name}/Images/")
            shutil.copytree(self.project.p_project+"/Icons/",f"{self.project.p_project}/dist/{self.project.title}-{self.version.strip(" ")}/Icons/",dirs_exist_ok=True)
            shutil.copytree(self.project.p_project+"/Images/",f"{self.project.p_project}/dist/{self.project.title}-{self.version.strip(" ")}/Images/",dirs_exist_ok=True)
        print(f"\n\nReleased new{self.version}build of {self.project.title}!")

    def newVersion(self):
        """Updates the project version, PERMANENT, cannot be undone
        """
        old = str(self.project.version)
        vers = self.project.version.split(".")
        if self.version == "Major":
            vers[0] = str(int(vers[0])+1)
            vers[1] = str(0)
            vers[2] = str(0)
        if self.version == "Minor":
            vers[1] = str(int(vers[1])+1)
            vers[2] = str(0)
        if self.version == "Patch":
            vers[2] = str(int(vers[2])+1)

        self.project.setVersion(f"{vers[0]}.{vers[1]}.{vers[2]}")
        self.project = VINFO()
        print(f"Updated Version {old}=>{self.project.version}")

    def makeRelease(self):
        """Releases a version of your project
        """
        match self.version:
            case "a":
                self.build("alpha")
                subprocess.call(f"pyinstaller {self.project.p_vinfo}/project.spec --noconfirm --distpath {self.project.p_project}/dist/ --log-level FATAL")
                self.clean(" alpha ")
            case "b":
                self.build("beta")
                subprocess.call(f"pyinstaller {self.project.p_vinfo}/project.spec --noconfirm --distpath {self.project.p_project}/dist/ --log-level FATAL")
                self.clean(" beta ")
            case "c":
                self.newVersion(type)
                self.build()
                subprocess.call(f"pyinstaller {self.project.p_vinfo}/project.spec --noconfirm --distpath {self.project.p_project}/dist/ --log-level FATAL")
                self.clean()
            case "sync":
                self.build("alpha")
                subprocess.call(f"pyinstaller {self.project.p_vinfo}/project.spec --noconfirm --distpath {self.project.p_project}/dist/ --log-level FATAL")
                self.clean(" alpha ")
                self.build("beta")
                subprocess.call(f"pyinstaller {self.project.p_vinfo}/project.spec --noconfirm --distpath {self.project.p_project}/dist/ --log-level FATAL")
                self.clean(" beta ")
                self.build()
                subprocess.call(f"pyinstaller {self.project.p_vinfo}/project.spec --noconfirm --distpath {self.project.p_project}/dist/ --log-level FATAL")
                self.clean()
                print("\t- alpha\n\t- beta\n\t- current")
            case _:
                print(f"Could not release Project Version {self.version}")


from typing import Literal
from tkinter import *
from tkinter import ttk
from VIStk.Objects import SubRoot

class QuestionWindow(SubRoot):
    """An empty popout window"""
    def __init__(self, question:str|list[str], answer:str, parent:Toplevel|Tk, ycommand=None, droplist:list=None, *args,**kwargs):
        """Will create a question window
            y = yes
            n = no
            r = return
            u = continue
            b = back
            x = close
            c = confirm
            d = dropdown
        """
        super().__init__(*args,**kwargs)
        cs = len(list(answer))

        for i in range(0,cs,1):
            self.columnconfigure(i,weight=1)

        #Resolve Question
        if isinstance(question, str):
            self.rowconfigure(0, weight=1)
            rs = 1
            Label(self, text=question, anchor="w").grid(row=0,column=0,columnspan=cs,sticky=(N,S,E,W))
        else:
            rs = len(question)
            for i in range(0,rs,1):
                self.rowconfigure(i, weight=1)
                Label(self, text=question[i], anchor="w").grid(row=i,column=0,columnspan=cs,sticky=(N,S,E,W))
        self.rowconfigure(rs, weight=1)

        #Resolve Answer
        self.elements = list(answer)

        self.screen_elements = []
        for i in range(0,len(self.elements),1):
            match self.elements[i]:
                case "y":
                    self.screen_elements.append(Button(self, text="Yes", command = lambda: self.ycom(ycommand)))
                    self.screen_elements[i].grid(row=rs,column=i,sticky=(N,S,E,W))
                case "n":
                    self.screen_elements.append(Button(self, text="No", command = self.xcom))
                    self.screen_elements[i].grid(row=rs,column=i,sticky=(N,S,E,W))
                case "r":
                    self.screen_elements.append(Button(self, text="Return", command = self.xcom))
                    self.screen_elements[i].grid(row=rs,column=i,sticky=(N,S,E,W))
                case "u":
                    self.screen_elements.append(Button(self, text="Continue", command = lambda: self.ycom(ycommand)))
                    self.screen_elements[i].grid(row=rs,column=i,sticky=(N,S,E,W))
                case "b":
                    self.screen_elements.append(Button(self, text="Back", command = self.xcom))
                    self.screen_elements[i].grid(row=rs,column=i,sticky=(N,S,E,W))
                case "x":
                    self.screen_elements.append(Button(self, text="Close", command = self.xcom))
                    self.screen_elements[i].grid(row=rs,column=i,sticky=(N,S,E,W))
                case "c":
                    self.screen_elements.append(Button(self, text="Confirm", command = lambda: self.ycom(ycommand)))
                    self.screen_elements[i].grid(row=rs,column=i,sticky=(N,S,E,W))
                case "d":
                    self.screen_elements.append(ttk.Combobox(self, values=droplist))
                    self.screen_elements[i].grid(row=rs,column=i,sticky=(N,S,E,W))
                case _:
                    pass


        #Ensure visibility
        self.focus_force()
        
        #SubWindow Geometry
        self.update()
        self.WindowGeometry.getGeometry(True)
        self.WindowGeometry.setGeometry(width=self.winfo_width(),
                                        height=self.winfo_height(),
                                        align="center",
                                        size_style="window_relative",
                                        window_ref=parent)

    def ycom(self,command):
        self.destroy()
        if not command is None:
            command()

    def xcom(self):
        self.destroy()


from VIStk.Objects._WindowGeometry import *
from VIStk.Objects._Root import *
from VIStk.Objects._SubRoot import *
from VIStk.Objects._Window import *

__all__ = ["WindowGeometry","Root","SubRoot","Window"]


import json
from tkinter import *
from tkinter import ttk
from VIStk.Widgets import MenuItem

class VISMenu():
    """The menu class drawings a column of buttons with subprocess calls to paths defined in a corresponding .json file.

    Has two roots because can destory both main window and subwindow on redirect.
    """
    def __init__(self, parent:Frame|LabelFrame|Toplevel|Tk, path:str):
        """
        Args:
            root (Tk): Master root for destruction on redirect
            _root (Toplevel): Toplevel object to create menu on
            path (str): Path to .json file describing menu
            destroyOnRedirect (bool): If True the root window will be destroyed on redicet
        """

        self.parent = parent
        self.root = self.parent.winfo_toplevel()
        self.path = path
        self.ob_dict = []
        self.n_dict = {}

        #Open json file for menu structure
        with open(path) as file:
            self.dict:dict = json.load(file)
        self.parent.grid_columnconfigure(0,weight=1)
        for i in range(0, len(self.dict.keys()), 1):
            self.parent.grid_rowconfigure(i,weight=1)

        x = 0
        for item in self.dict:
            ob = MenuItem(self.parent,
                      path= self.dict[item]["path"],
                      nav = self.dict[item]["nav"],
                      text = self.dict[item]["text"],
                      relief="flat"
                      )
            ob.button.grid(row=x, column=0, sticky=(N,S,E,W))
            self.ob_dict.append(ob)
            self.n_dict[ob.nav]=ob
            x += 1

        self.root.bind("<KeyPress>",self.menuNav)
    
    def menuNav(self,happ:Event):
        k=happ.char
        if self.n_dict.get(k) != None:
            self.n_dict[k].itemPath()



from VIStk.Structures.project import *
from VIStk.Structures.release import *
from VIStk.Structures.screen import *
from VIStk.Structures.VINFO import *

__all__ = ["Project", "Release", "Screen", "VINFO"]


from tkinter import *
from VIStk.Objects import *
from VIStk.Widgets import *
import time

root = Root()
root.WindowGeometry.setGeometry(width=40,height=40,align="center",size_style="screen_relative")

def warnWindow():
    newroot = WarningWindow(parent=root,
                             warning="Warning: Something!!!",
                            )

def subWindow():
    subroot = QuestionWindow(parent=root,
                             question=["Some info...",
                                       "Additional information given",
                                       "Do you want to continue"],
                             answer = "xu",
                             ycommand=warnWindow
                            ).fullscreen()

vb_test = Button(root, text="Open Submenu", command=subWindow)
vb_test.grid(row=1,column=1,sticky=(N, S, E, W))

root.mainloop()